@@ -9,7 +9,7 @@ from random import randint
 from sys import modules
 from typing import Dict, Optional
 
-from .const import ATTR_ALIAS, ATTR_CLASS, ATTR_FAKED, Discover, __dev_mode__
+from .const import ATTR_ALIAS, ATTR_CLASS, ATTR_FAKED, DHW_HACK, Discover, __dev_mode__
 from .entities import Entity, discover_decorator
 from .protocol import Command, Priority  # TODO: constants to const.py
 from .protocol.address import NON_DEV_ADDR, id_to_address
@@ -21,13 +21,24 @@ from .protocol.const import (
     ATTR_SETPOINT,
     ATTR_TEMP,
     ATTR_WINDOW_OPEN,
+    BOOST_TIMER,
     DEVICE_CLASS,
     DEVICE_TYPES,
     DOMAIN_TYPE_MAP,
+    FAN_MODE,
     NUL_DEVICE_ID,
 )
 from .protocol.exceptions import CorruptStateError
-from .protocol.opentherm import MSG_ID, MSG_NAME, MSG_TYPE, OPENTHERM_MESSAGES, VALUE
+from .protocol.opentherm import (
+    MSG_ID,
+    MSG_NAME,
+    MSG_TYPE,
+    OPENTHERM_MESSAGES,
+    PARAMS_MSG_IDS,
+    SCHEMA_MSG_IDS,
+    STATUS_MSG_IDS,
+    VALUE,
+)
 from .protocol.ramses import CODE_ONLY_FROM_CTL, RAMSES_DEVICES
 from .protocol.transport import PacketProtocolPort
 
@@ -80,6 +91,7 @@ from .protocol import (  # noqa: F401, isort: skip
     _22F3,
     _2309,
     _2349,
+    _2389,
     _2D49,
     _2E04,
     _30C9,
@@ -211,7 +223,8 @@ class DeviceBase(Entity):
 
         if self._ctl is ctl:
             return
-        if self._is_controller:  # HACK
+        if self._is_controller and self.type != "02":  # DEX
+            # HACK: UFC is/binds to a contlr
             return
         if self._ctl is not None:
             raise CorruptStateError(f"{self} changed controller: {self._ctl} to {ctl}")
@@ -228,6 +241,8 @@ class DeviceBase(Entity):
         return ctl
 
     def _handle_msg(self, msg) -> None:
+        if DHW_HACK and msg.code == _1260:
+            _LOGGER.debug(f"{msg._pkt} < handling (11)")  # HACK: lloyda
         assert msg.src is self, f"msg inappropriately routed to {self}"
         super()._handle_msg(msg)
 
@@ -328,16 +343,18 @@ class Device(DeviceInfo, DeviceBase):
     DEVICE_TYPES = tuple()
 
     def _handle_msg(self, msg) -> None:
+        if DHW_HACK and msg.code == _1260:
+            _LOGGER.debug(f"{msg._pkt} < handling (10)")  # HACK: lloyda
         super()._handle_msg(msg)
 
         if type(self) is Device and self.type == "30":  # self.__class__ is Device, DEX
             # TODO: the RFG codes need checking
             if msg.code in (_31D9, _31DA, _31E0) and msg.verb in (I_, RP):
-                self.__class__ = FanDevice
+                self.__class__ = HvacVentilator
             elif msg.code in (_0006, _0418, _3220) and msg.verb == RQ:
-                self.__class__ = RFGateway
+                self.__class__ = RfgGateway
             elif msg.code in (_313F,) and msg.verb == W_:
-                self.__class__ = RFGateway
+                self.__class__ = RfgGateway
 
         if not msg._gwy.config.enable_eavesdrop:
             return
@@ -789,14 +806,14 @@ class RelayDemand(Fakeable):  # 0008 (fakeable)
         }
 
 
-class RFGateway(DeviceInfo, DeviceBase):  # RFG (30:)
+class RfgGateway(DeviceInfo, DeviceBase):  # RFG (30:)
     """The RFG100 base class."""
 
     _class = DEVICE_CLASS.RFG
     _types = ("30",)
 
 
-class HGInterface(DeviceBase):  # HGI (18:), was GWY
+class HgiGateway(DeviceBase):  # HGI (18:), was GWY
     """The HGI80 base class."""
 
     _class = DEVICE_CLASS.HGI
@@ -1006,14 +1023,14 @@ class UfhController(Device):  # UFC (02):
         if discover_flag & Discover.SCHEMA:
             [  # 0005: shows which channels are active - ?no use? (see above)
                 self._make_cmd(_0005, payload=f"00{zone_type}")
-                for zone_type in ("09",)  # _0005_ZONE_TYPE, also ("00", "04", "0F")
+                for zone_type in ("00", "04", "09", "0F")  # TODO: are all req'd
                 # for zone_type in _0005_ZONE_TYPE
             ]
 
             [  # 000C: used to find evo zone for each configured channel
-                self._make_cmd(_000C, payload=f"{idx:02X}{_000C_DEVICE.UFH}")
+                self._make_cmd(_000C, payload=f"{idx:02X}{_000C_DEVICE.ALL}")
                 for idx in range(8)  # for each possible UFH channel/circuit
-            ]
+            ]  # _000C_DEVICE.UFH doesn't seem to work with all UFCs
 
         # if discover_flag & Discover.STATUS:
         #     [  # 22C9: no answer
@@ -1146,10 +1163,18 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
         return f"{self.id} ({self._domain_id}): {self.temperature}"
 
     def _handle_msg(self, msg) -> None:  # NOTE: active
+        if DHW_HACK:
+            _LOGGER.debug(f"{msg._pkt} < handling (00)")  # HACK: lloyda
+            _LOGGER.debug(f"msgs = {self._msgs}")
+
         super()._handle_msg(msg)
 
+        if DHW_HACK:
+            _LOGGER.debug(f"{msg._pkt} < handled. (00)")  # HACK: lloyda
+            _LOGGER.debug(f"msgs = {self._msgs}")
+
         if msg.code == _1260 and self._ctl and not self._gwy.config.disable_sending:
-            # device can be instatiated with a CTL
+            # device can be instantiated with a CTL
             self._send_cmd(Command.get_dhw_temp(self._ctl.id))
 
     @property
@@ -1158,7 +1183,32 @@ class DhwSensor(BatteryState, Device):  # DHW (07): 10A0, 1260
 
     @property
     def temperature(self) -> Optional[float]:  # 1260
-        return self._msg_value(_1260, key=self.TEMPERATURE)
+        result = None
+        try:  # HACK: lloyda
+            result = self._msg_value(_1260, key=self.TEMPERATURE)
+        except (
+            ArithmeticError,  # incl. ZeroDivisionError,
+            AssertionError,
+            AttributeError,
+            IndexError,
+            LookupError,  # incl. IndexError, KeyError
+            NameError,  # incl. UnboundLocalError
+            RuntimeError,  # incl. RecursionError
+            TypeError,
+            ValueError,
+        ) as exc:
+            _LOGGER.exception(exc)
+
+        if DHW_HACK and result is None:  # HACK: lloyda
+            msg = self._msgs.get(_1260)
+            _LOGGER.info(f"DHW msg 1 = {msg!r}")
+            if msg:
+                result = msg.payload.get(self.TEMPERATURE)
+            else:
+                _LOGGER.info(f"DHW msg 2 = {msg!r}")
+                _LOGGER.debug(f"msgs = {self._msgs}")
+                _LOGGER.debug(f"msgz = {self._msgz}")
+        return result
 
     @property
     def params(self) -> dict:
@@ -1201,56 +1251,6 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
     # OPENTHERM_STATUS = "opentherm_status"
     # _STATE = super().MODULATION_LEVEL
 
-    SCHEMA_MSG_IDS = (
-        "03",  # ..3: "Slave configuration",
-        "06",  # ..6: "Remote boiler parameter flags",                      # 0x38, 0x39
-        "0F",  # .15: "Max. boiler capacity (kW) and modulation level setting (%)",
-        "30",  # .48: "DHW Setpoint upper & lower bounds for adjustment (°C)",
-        "31",  # .49: "Max CH water Setpoint upper & lower bounds for adjustment (°C)",
-        "7D",  # 125: "Opentherm version Slave",                            # not native
-        "7F",  # 127: "Slave product version number and type",
-    )
-    PARAMS_MSG_IDS = (
-        "38",  # .56: "DHW Setpoint (°C) (Remote parameter 1)",             # see: 0x06
-        "39",  # .57: "Max CH water Setpoint (°C) (Remote parameter 2)",    # see: 0x06
-        # These are error codes...
-        "05",  # ..5: "Fault flags & OEM codes",
-        "73",  # 115: "OEM diagnostic code",
-        # These are STATUS seen RQ'd by 01:/30:, but here to retreive less frequently
-        "71",  # 113: "Number of un-successful burner starts",
-        "72",  # 114: "Number of times flame signal was too low",
-        "74",  # 116: "Number of starts burner",
-        "75",  # 117: "Number of starts central heating pump",
-        "76",  # 118: "Number of starts DHW pump/valve",
-        "77",  # 119: "Number of starts burner during DHW mode",
-        "78",  # 120: "Number of hours burner is in operation (i.e. flame on)",
-        "79",  # 121: "Number of hours central heating pump has been running",
-        "7A",  # 122: "Number of hours DHW pump has been running/valve has been opened",
-        "7B",  # 123: "Number of hours DHW burner is in operation during DHW mode",
-    )
-    STATUS_MSG_IDS = (
-        "00",  # ..0: "Master/Slave status flags",                          # not native
-        "01",  # ..1: "CH water temperature Setpoint (°C)",                 # also R/W
-        "11",  # .17: "Relative Modulation Level (%)",
-        "12",  # .18: "Water pressure in CH circuit (bar)",
-        "13",  # .19: "Water flow rate in DHW circuit. (L/min)",
-        "19",  # .25: "Boiler flow water temperature (°C)",
-        "1A",  # .26: "DHW temperature (°C)",
-        "1B",  # .27: "Outside temperature (°C)",  # TODO: any value here?  # not native
-        "1C",  # .28: "Return water temperature (°C)",
-    )
-    WRITE_MSG_IDS = (  # Write-Data, NB: some are also Read-Data
-        "01",  # ..1:  -see above-
-        "02",  # ..2: "Master configuration",
-        "0E",  # .14: "Maximum relative modulation level setting (%)",  # c.f. 0x11
-        "10",  # .16: "Room Setpoint (°C)",     # tell slave the room setpoint?
-        "18",  # .24: "Room temperature (°C)",  # tell slave the room temp?
-        "38",  # .56:  -see above-
-        "39",  # .57:  -see above-
-        "7C",  # 124: "Opentherm version Master",
-        "7E",  # 126: "Master product version number and type",
-    )
-
     def __init__(self, *args, **kwargs) -> None:
         super().__init__(*args, **kwargs)
 
@@ -1272,7 +1272,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         if discover_flag & Discover.SCHEMA:
             [
                 self._send_cmd(Command.get_opentherm_data(self.id, m))
-                for m in self.SCHEMA_MSG_IDS  # From OT v2.2: version numbers
+                for m in SCHEMA_MSG_IDS  # From OT v2.2: version numbers
                 if self._supported_msg.get(m) is not False
                 and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]
@@ -1280,7 +1280,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         if discover_flag & Discover.PARAMS:
             [
                 self._send_cmd(Command.get_opentherm_data(self.id, m))
-                for m in self.PARAMS_MSG_IDS
+                for m in PARAMS_MSG_IDS
                 if self._supported_msg.get(m) is not False
                 and (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]
@@ -1303,7 +1303,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
             self._send_cmd(Command(RQ, _12F0, "00", self.id))
             [
                 self._send_cmd(Command.get_opentherm_data(self.id, m, retries=0))
-                for m in self.STATUS_MSG_IDS
+                for m in STATUS_MSG_IDS
                 if self._supported_msg.get(m) is not False
                 # nd (not self._opentherm_msg.get(m) or self._opentherm_msg[m]._expired)
             ]  # TODO: add expired
@@ -1327,7 +1327,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
             elif self._supported_msg[msg_id] is None:
                 self._supported_msg[msg_id] = False
                 _LOGGER.warning(
-                    f"{msg._pkt} << OpenTherm: deprecating msg_id "
+                    f"{msg._pkt} < OpenTherm: deprecating msg_id "
                     f"0x{msg_id}: it appears unsupported",
                 )
 
@@ -1401,7 +1401,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         result = {
             self._msg_name(v): v.payload
             for k, v in self._opentherm_msg.items()
-            if self._supported_msg.get(int(k, 16)) and int(k, 16) in self.SCHEMA_MSG_IDS
+            if self._supported_msg.get(int(k, 16)) and int(k, 16) in SCHEMA_MSG_IDS
         }
         return {
             m: {k: v for k, v in p.items() if k.startswith(VALUE)}
@@ -1413,7 +1413,7 @@ class OtbGateway(Actuator, HeatDemand, Device):  # OTB (10): 3220 (22D9, others)
         result = {
             self._msg_name(v): v.payload
             for k, v in self._opentherm_msg.items()
-            if self._supported_msg.get(int(k, 16)) and int(k, 16) in self.PARAMS_MSG_IDS
+            if self._supported_msg.get(int(k, 16)) and int(k, 16) in PARAMS_MSG_IDS
         }
         return {
             m: {k: v for k, v in p.items() if k.startswith(VALUE)}
@@ -1635,7 +1635,7 @@ class TrvActuator(BatteryState, HeatDemand, Setpoint, Temperature, Device):  # T
     """The TRV class, such as a HR92."""
 
     _class = DEVICE_CLASS.TRV
-    _types = ("00", "04")
+    _types = ("00", "04")  # TODO: keep 00?
 
     WINDOW_OPEN = ATTR_WINDOW_OPEN  # boolean
     # _STATE = HEAT_DEMAND
@@ -1655,7 +1655,42 @@ class TrvActuator(BatteryState, HeatDemand, Setpoint, Temperature, Device):  # T
         }
 
 
-class FanSwitch(BatteryState, Device):  # SWI (39): I/22F[13]
+class HvacHumidity(BatteryState, Device):  # HUM (32) I/12(98|A0)
+    """The Sensor class for a humidity sensor.
+
+    The cardinal code is 12A0.
+    """
+
+    _class = DEVICE_CLASS.HUM
+    _types = tuple()  # ("32",)
+
+    REL_HUMIDITY = "relative_humidity"  # percentage (0.0-1.0)
+    TEMPERATURE = "temperature"  # celsius
+    DEWPOINT_TEMP = "dewpoint_temp"  # celsius
+
+    @property
+    def relative_humidity(self) -> Optional[float]:
+        return self._msg_value(_12A0, key=self.REL_HUMIDITY)
+
+    @property
+    def temperature(self) -> Optional[float]:
+        return self._msg_value(_12A0, key=self.TEMPERATURE)
+
+    @property
+    def dewpoint_temp(self) -> Optional[float]:
+        return self._msg_value(_12A0, key=self.DEWPOINT_TEMP)
+
+    @property
+    def status(self) -> dict:
+        return {
+            **super().status,
+            self.REL_HUMIDITY: self.relative_humidity,
+            self.TEMPERATURE: self.temperature,
+            self.DEWPOINT_TEMP: self.dewpoint_temp,
+        }
+
+
+class HvacSwitch(BatteryState, Device):  # SWI (39): I/22F[13]
     """The FAN (switch) class, such as a 4-way switch.
 
     The cardinal codes are 22F1, 22F3.
@@ -1666,40 +1701,26 @@ class FanSwitch(BatteryState, Device):  # SWI (39): I/22F[13]
     # RP --- 30:079129 32:166025 --:------ 31DA 029 21EF00026036EF7FFF7FFF7FFF7FFF0002EF18FFFF000000EF7FFF7FFF
 
     _class = DEVICE_CLASS.SWI
-    _types = ("39",)
-
-    BOOST_TIMER = "boost_timer"  # minutes, e.g. 10, 20, 30 minutes
-    HEATER_MODE = "heater_mode"  # e.g. auto, off
-    HEATER_MODES = {9: "off", 10: "auto"}  # TODO:
-
-    FAN_MODE = "fan_mode"  # e.g. low. high
-    FAN_MODES = {
-        0: "standby",
-        1: "auto",
-        2: "low",
-        3: "medium",
-        4: "high",  # a.k.a. boost if timer on
-    }
-    FAN_RATE = "fan_rate"  # percentage, 0.0 - 1.0
+    _types = tuple()  # ("39",)
 
     @property
     def fan_mode(self) -> Optional[str]:
-        return self._msg_value(_22F1, key=self.FAN_MODE)
+        return self._msg_value(_22F1, key=FAN_MODE)
 
     @property
     def boost_timer(self) -> Optional[int]:
-        return self._msg_value(_22F3, key=self.BOOST_TIMER)
+        return self._msg_value(_22F3, key=BOOST_TIMER)
 
     @property
     def status(self) -> dict:
         return {
             **super().status,
-            self.FAN_MODE: self.fan_mode,
-            self.BOOST_TIMER: self.boost_timer,
+            FAN_MODE: self.fan_mode,
+            BOOST_TIMER: self.boost_timer,
         }
 
 
-class FanDevice(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
+class HvacVentilator(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
     """The Ventilation class.
 
     The cardinal code are 31D9, 31DA.  Signature is RP/31DA.
@@ -1713,7 +1734,7 @@ class FanDevice(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
     # 30:079129 --:------ 30:079129 31D9 017 2100FF0000000000000000000000000000
 
     _class = DEVICE_CLASS.FAN
-    _types = ("20", "37")
+    _types = tuple()  # ("20", "37")
 
     @property
     def fan_rate(self) -> Optional[float]:
@@ -1753,41 +1774,6 @@ class FanDevice(Device):  # FAN (20/37): RP/31DA, I/31D[9A]
         }
 
 
-class FanSensorHumidity(BatteryState, Device):  # HUM (32) I/12(98|A0)
-    """The Sensor class for a humidity sensor.
-
-    The cardinal code is 12A0.
-    """
-
-    _class = DEVICE_CLASS.HUM
-    _types = ("32",)
-
-    REL_HUMIDITY = "relative_humidity"  # percentage (0.0-1.0)
-    TEMPERATURE = "temperature"  # celsius
-    DEWPOINT_TEMP = "dewpoint_temp"  # celsius
-
-    @property
-    def relative_humidity(self) -> Optional[float]:
-        return self._msg_value(_12A0, key=self.REL_HUMIDITY)
-
-    @property
-    def temperature(self) -> Optional[float]:
-        return self._msg_value(_12A0, key=self.TEMPERATURE)
-
-    @property
-    def dewpoint_temp(self) -> Optional[float]:
-        return self._msg_value(_12A0, key=self.DEWPOINT_TEMP)
-
-    @property
-    def status(self) -> dict:
-        return {
-            **super().status,
-            self.REL_HUMIDITY: self.relative_humidity,
-            self.TEMPERATURE: self.temperature,
-            self.DEWPOINT_TEMP: self.dewpoint_temp,
-        }
-
-
 _CLASS = "_class"
 DEVICE_BY_CLASS = {
     getattr(c[1], _CLASS): c[1]
@@ -1795,7 +1781,7 @@ DEVICE_BY_CLASS = {
         modules[__name__],
         lambda m: isclass(m) and m.__module__ == __name__ and hasattr(m, _CLASS),
     )
-}  # e.g. "CTL": Controller
+}  # Every device class has a {klass: class}, e.g.: "CTL": Controller
 
 DEVICE_BY_ID_TYPE = {
     k1: v2
