@@ -177,9 +177,14 @@ def process_msg(msg: Message) -> None:
             msg._gwy._prev_msg = msg
             return
 
+        if msg.src.type == "07":
+            _LOGGER.debug(f"{msg._pkt} < handling (01)")  # HACK: lloyda
+
         # _update_entities(msg, msg._gwy._prev_msg)  # update the state database
         if isinstance(msg.src, Device):
             msg.src._handle_msg(msg)
+        else:
+            _LOGGER.debug(f"{msg._pkt} < handling (02)")  # HACK: lloyda
 
         if msg.code not in (_0008, _0009, _3B00, _3EF1):  # special case: are fakeable
             msg._gwy._prev_msg = msg
@@ -201,18 +206,18 @@ def process_msg(msg: Message) -> None:
 
     except (AssertionError, NotImplementedError) as exc:
         (_LOGGER.error if DEV_MODE else _LOGGER.warning)(
-            "%s << %s", msg._pkt, f"{exc.__class__.__name__}({exc})"
+            "%s < %s", msg._pkt, f"{exc.__class__.__name__}({exc})"
         )
         return  # NOTE: use raise only when debugging
 
     except (AttributeError, LookupError, TypeError, ValueError) as exc:
         (_LOGGER.exception if DEV_MODE else _LOGGER.error)(
-            "%s << %s", msg._pkt, f"{exc.__class__.__name__}({exc})"
+            "%s < %s", msg._pkt, f"{exc.__class__.__name__}({exc})"
         )
         return  # NOTE: use raise only when debugging
 
     except CorruptStateError as exc:  # TODO: add CorruptEvohomeError
-        (_LOGGER.exception if DEV_MODE else _LOGGER.error)("%s << %s", msg._pkt, exc)
+        (_LOGGER.exception if DEV_MODE else _LOGGER.error)("%s < %s", msg._pkt, exc)
         return  # TODO: bad pkt, or Schema
 
     msg._gwy._prev_msg = msg
