@@ -6,7 +6,10 @@
 import re
 from types import SimpleNamespace
 
-DEV_MODE = __dev_mode__ = False  # True
+__dev_mode__ = False
+DEV_MODE = __dev_mode__
+
+DHW_HACK = True
 
 
 def slug(string: str) -> str:
@@ -32,6 +35,28 @@ DEVICE_CLASS = SimpleNamespace(
     UFC="UFC",  # UFH controller
 )
 
+DEVICE_CLASS_BY_TYPE = {
+    "01": DEVICE_CLASS.CTL,
+    "02": DEVICE_CLASS.UFC,
+    "04": DEVICE_CLASS.TRV,
+    "07": DEVICE_CLASS.DHW,
+    "10": DEVICE_CLASS.OTB,
+    "13": DEVICE_CLASS.BDR,
+    "17": DEVICE_CLASS.EXT,
+    "18": DEVICE_CLASS.HGI,
+    "23": DEVICE_CLASS.PRG,
+    "34": DEVICE_CLASS.STA,
+}
+DEVICE_TYPE_BY_CLASS = {v: k for k, v in DEVICE_CLASS_BY_TYPE.items()}
+DEVICE_CLASS_BY_TYPE.update(
+    {
+        "00": DEVICE_CLASS.TRV,
+        "03": DEVICE_CLASS.STA,
+        "12": DEVICE_CLASS.STA,
+        "22": DEVICE_CLASS.STA,
+    }
+)
+
 HGI_DEVICE_ID = "18:000730"  # default type and address of HGI, 18:013393
 NON_DEVICE_ID = "--:------"
 NUL_DEVICE_ID = "63:262142"  # FFFFFE - send here if not bound?
@@ -51,6 +76,7 @@ _000C = "000C"
 _000E = "000E"
 _0016 = "0016"
 _0100 = "0100"
+_0150 = "0150"
 _01D0 = "01D0"
 _01E9 = "01E9"
 _0404 = "0404"
@@ -59,9 +85,13 @@ _042F = "042F"
 _0B04 = "0B04"
 _1030 = "1030"
 _1060 = "1060"
+_1081 = "1081"
 _1090 = "1090"
+_1098 = "1098"
 _10A0 = "10A0"
+_10B0 = "10B0"
 _10E0 = "10E0"
+_10E1 = "10E1"
 _1100 = "1100"
 _1260 = "1260"
 _1280 = "1280"
@@ -71,9 +101,12 @@ _12A0 = "12A0"
 _12B0 = "12B0"
 _12C0 = "12C0"
 _12C8 = "12C8"
+_12F0 = "12F0"
+_1300 = "1300"
 _1F09 = "1F09"
 _1F41 = "1F41"
 _1FC9 = "1FC9"
+_1FD0 = "1FD0"
 _1FD4 = "1FD4"
 _2249 = "2249"
 _22C9 = "22C9"
@@ -83,6 +116,11 @@ _22F1 = "22F1"
 _22F3 = "22F3"
 _2309 = "2309"
 _2349 = "2349"
+_2389 = "2389"
+_2400 = "2400"
+_2401 = "2401"
+_2410 = "2410"
+_2420 = "2420"
 _2D49 = "2D49"
 _2E04 = "2E04"
 _30C9 = "30C9"
@@ -92,7 +130,11 @@ _3150 = "3150"
 _31D9 = "31D9"
 _31DA = "31DA"
 _31E0 = "31E0"
+_3200 = "3200"
+_3210 = "3210"
 _3220 = "3220"
+_3221 = "3221"
+_3223 = "3223"
 _3B00 = "3B00"
 _3EF0 = "3EF0"
 _3EF1 = "3EF1"
@@ -131,174 +173,101 @@ DEVICE_ID_REGEX = SimpleNamespace(
     SEN=DEV_REGEX_SEN,
 )
 
-# Packet codes (this dict is being deprecated) - check against ramses.py
-CODE_SCHEMA = {
-    _0001: {"uses_zone_idx": True},
-    _01D0: {"uses_zone_idx": True},
-    _01E9: {"uses_zone_idx": True},
-    _0404: {"uses_zone_idx": True},
-    _3EF1: {"uses_zone_idx": True},
-    _0004: {"uses_zone_idx": True},
-    _0008: {"uses_zone_idx": True},
-    _0009: {"uses_zone_idx": True},
-    _000A: {"uses_zone_idx": True},
-    _1030: {"uses_zone_idx": True},
-    _1060: {"uses_zone_idx": True},
-    _12B0: {"uses_zone_idx": True},
-    _1FC9: {"uses_zone_idx": True},
-    _2249: {"uses_zone_idx": True},
-    _2309: {"uses_zone_idx": True},
-    _2349: {"uses_zone_idx": True},
-    _30C9: {"uses_zone_idx": True},
-    _3150: {"uses_zone_idx": True},
-}
+# # Packet codes (this dict is being deprecated) - check against ramses.py
+# CODE_SCHEMA = {
+#     _0001: {"uses_zone_idx": True},
+#     _01D0: {"uses_zone_idx": True},
+#     _01E9: {"uses_zone_idx": True},
+#     _0404: {"uses_zone_idx": True},
+#     _3EF1: {"uses_zone_idx": True},
+#     _0004: {"uses_zone_idx": True},
+#     _0008: {"uses_zone_idx": True},
+#     _0009: {"uses_zone_idx": True},
+#     _000A: {"uses_zone_idx": True},
+#     _1030: {"uses_zone_idx": True},
+#     _1060: {"uses_zone_idx": True},
+#     _12B0: {"uses_zone_idx": True},
+#     _1FC9: {"uses_zone_idx": True},
+#     _2249: {"uses_zone_idx": True},
+#     _2309: {"uses_zone_idx": True},
+#     _2349: {"uses_zone_idx": True},
+#     _30C9: {"uses_zone_idx": True},
+#     _3150: {"uses_zone_idx": True},
+# }
 
-_MAY_USE_ZONE_IDX = [k for k, v in CODE_SCHEMA.items() if v.get("uses_zone_idx")]
+# _MAY_USE_ZONE_IDX = [k for k, v in CODE_SCHEMA.items() if v.get("uses_zone_idx")]
 
 DEVICE_TABLE = {
     # Honeywell evohome
     "01": {
         "type": "CTL",
         "name": "Controller",
-        "has_battery": False,
-        "has_zone_sensor": False,  # a special case
-        "is_actuator": False,
-        "is_controller": True,
-        "is_sensor": True,
         "archetype": "ATC928",
-        "poll_codes": [_000C, _10E0, _1100, _313F],
-        "discover_schema": [],
     },  # rechargeable
     "02": {
         "type": "UFC",
         "name": "UFH Controller",
-        "has_battery": False,
-        "is_actuator": None,
-        "is_controller": None,
-        "is_sensor": None,
         "archetype": "HCE80(R)",
-        "discover_schema": [],
     },
     "03": {
         "type": "STa",
         "name": "Room Sensor/Stat",
-        "has_battery": True,
-        "has_zone_sensor": True,
-        "is_actuator": False,
-        "is_sensor": True,
         "archetype": "HCW82",  # also: HCF82
-        "discover_schema": [],
     },
     "04": {
         "type": "TRV",
         "name": "Radiator Valve",
-        "has_battery": True,
-        "has_zone_sensor": True,
-        "is_actuator": True,
-        "is_sensor": True,
         "archetype": "HR92",  # also: HR80
-        "discover_schema": [],
     },  #
     "07": {
         "type": "DHW",
         "name": "DHW Sensor",
-        "has_battery": True,
-        "is_actuator": False,
-        "is_sensor": True,
         "archetype": "CS92A",
-        "discover_schema": [],
     },
     "10": {
         "type": "OTB",
         "name": "OpenTherm Bridge",
-        "has_battery": False,
-        "is_actuator": None,
-        "is_sensor": False,
         "archetype": "R8810",
-        "poll_codes": [
-            _0008,
-            _10A0,
-            _1100,
-            _1260,
-            _1290,
-            _22D9,
-            _3150,
-            _3220,
-            _3EF0,
-            _3EF1,
-        ],
-        "discover_schema": [],
     },  #
     "13": {
         "type": "BDR",
         "name": "Wireless Relay",
-        "has_battery": False,
-        "is_actuator": None,
-        "is_sensor": False,
         "archetype": "BDR91",  # also: HC60NG?
-        "poll_codes": [_0008, _1100, _3EF1],
-        "discover_schema": [],  # excl.: 10E0
     },
     "22": {
         "type": "THM",
         "name": "Room Thermostat",
-        "has_battery": True,
-        "has_zone_sensor": True,
-        "is_actuator": False,
-        "is_sensor": True,
         "archetype": "DTS92(E)",
-        "discover_schema": [],
     },
     "30": {
         "type": "RFG",
         "name": "Internet Gateway",
-        "has_battery": False,
-        "is_actuator": False,
-        "is_sensor": False,
         "archetype": "-unclear-",  # RFG100, VMS?
-        "discover_schema": [],
     },
     "34": {
         "type": "STA",
         "name": "Round Thermostat",
-        "has_battery": True,
-        "has_zone_sensor": True,
-        "is_actuator": False,
-        "is_sensor": True,
         "archetype": "T87RF",
-        "discover_schema": [],
     },
     # Honeywell evohome TBD
     "x1": {
         "type": "HM8",
         "name": "Mixing Valve",
-        "has_battery": False,
-        "is_actuator": None,
-        "is_sensor": None,
         "archetype": "HM80",
     },  # TODO: ???
     # Honeywell, non-evohome
     "17": {
-        "type": " 17",
+        "type": "EXT",
         "name": "Outdoor Sensor?",
-        "has_battery": None,
-        "is_actuator": False,
-        "is_sensor": False,
     },  # TODO: HB85?
     "18": {
         "type": "HGI",
         "name": "Honeywell Gateway",
-        "has_battery": False,
-        "is_actuator": False,
-        "is_sensor": False,
         "archetype": "HGI80",
     },
     "23": {
         "type": "PRG",
         "name": "Programmer (wired)",
-        "has_battery": False,
-        "is_actuator": False,
-        "is_sensor": True,
         "archetype": "ST9420C",
     },
     # Honeywell Jasper, HVAC?
@@ -333,12 +302,12 @@ DEVICE_TYPES = {k: v["type"] for k, v in DEVICE_TABLE.items()}
 DEVICE_LOOKUP = {v: k for k, v in DEVICE_TYPES.items()}
 # DEVICE_CLASSES = {v["type"]: v["name"] for _, v in DEVICE_TABLE.items()}
 
-DEVICE_HAS_BATTERY = tuple(
-    k for k, v in DEVICE_TABLE.items() if v.get("has_battery") is True
-)  # more correctly: is battery-powered (and so won't respond to RQs)
-DEVICE_HAS_ZONE_SENSOR = tuple(
-    k for k, v in DEVICE_TABLE.items() if v.get("has_zone_sensor") is True
-)  # other sensors (e.g. 07:) can't be used as a zone sensor
+# DEVICE_HAS_BATTERY = tuple(
+#     k for k, v in DEVICE_TABLE.items() if v.get("has_battery") is True
+# )  # more correctly: is battery-powered (and so won't respond to RQs)
+# DEVICE_HAS_ZONE_SENSOR = tuple(
+#     k for k, v in DEVICE_TABLE.items() if v.get("has_zone_sensor") is True
+# )  # other sensors (e.g. 07:) can't be used as a zone sensor
 # DEVICE_IS_ACTUATOR = tuple(
 #     k for k, v in DEVICE_TABLE.items() if v.get("is_actuator") is True
 # )  # c.f. 000C packet
@@ -453,7 +422,8 @@ _0005_ZONE = SimpleNamespace(
     HTG="0F",  # Heating control domains
     RFG="10",  # RFG gateway
     ELE="11",  # Electrical zones  # TODO: no RP from older evohome
-)
+)  # 03, 05, 06, 07: & >11 - no response from an 01:
+
 # RP --- 01:054173 18:006402 --:------ 0005 004 00100000  # before adding RFG100
 #  I --- 01:054173 --:------ 01:054173 1FC9 012 0010E004D39D001FC904D39D
 #  W --- 30:248208 01:054173 --:------ 1FC9 012 0010E07BC9900012907BC990
@@ -467,8 +437,6 @@ _0005_ZONE = SimpleNamespace(
 
 _0005_ZONE_TYPE = {
     _0005_ZONE.ALL: "zone_actuators",
-    # "01": None,
-    # "02": None,
     _0005_ZONE.ALL_SENSOR: "zone_sensor",
     _0005_ZONE.RAD: ATTR_RAD_VALVE,
     _0005_ZONE.UFH: ATTR_UFH_HTG,
@@ -480,7 +448,7 @@ _0005_ZONE_TYPE = {
     _0005_ZONE.HTG: ATTR_HTG_CONTROL,
     _0005_ZONE.RFG: "internet_gateway",
     _0005_ZONE.ELE: ATTR_ELEC_HEAT,
-}  # 03, 05, 06, 07: & >11 - no response from 01:
+}
 
 # RP|zone_devices | 000E0... || {'domain_id': 'FA', 'device_class': 'dhw_actuator', 'devices': ['13:081807']}  # noqa
 # RP|zone_devices | 010E0... || {'domain_id': 'FA', 'device_class': 'dhw_actuator', 'devices': ['13:106039']}  # noqa
@@ -490,11 +458,7 @@ _000C_DEVICE_TYPE = {
     _000C_DEVICE.ALL: "zone_actuators",
     # "01": None,
     # "02": None,
-    # "03": None,  # no response
     _000C_DEVICE.ALL_SENSOR: ATTR_ZONE_SENSOR,  # 03:, 04:, 34: (if is 01:, will == [], as if no sensor)
-    # "05": None,  # no response
-    # "06": None,  # no response
-    # "07": None,  # no response
     _000C_DEVICE.RAD: "rad_actuators",
     _000C_DEVICE.UFH: "ufh_actuators",
     _000C_DEVICE.VAL: "val_actuators",
@@ -511,9 +475,10 @@ _000C_DEVICE_TYPE = {
 _0418_DEVICE_CLASS = {
     "00": "controller",
     "01": "sensor",
+    "02": "setpoint",
     "04": "actuator",  # if domain is FC, then "boiler_relay"
     "05": "dhw_sensor",  # not ATTR_DHW_SENSOR
-    "06": "remote_gateway",  # 30:185469
+    "06": "remote_gateway",
 }
 _0418_FAULT_STATE = {
     "00": "fault",
@@ -524,8 +489,10 @@ _0418_FAULT_TYPE = {
     "01": "system_fault",
     "03": "mains_low",
     "04": "battery_low",
+    "05": "battery_error",  # evotouch_battery_error
     "06": "comms_fault",
     "0A": "sensor_error",
+    "??": "bad_value",
 }
 
 SystemType = SimpleNamespace(
@@ -619,3 +586,18 @@ class SystemModes((AttrDict)):
 
 
 SYSTEM_MODE = SystemModes(_SYSTEM_MODES)
+
+# used by 22Fx parser, and FanSwitch devices
+BOOST_TIMER = "boost_timer"  # minutes, e.g. 10, 20, 30 minutes
+HEATER_MODE = "heater_mode"  # e.g. auto, off
+HEATER_MODES = {9: "off", 10: "auto"}  # TODO:
+
+FAN_MODE = "fan_mode"  # e.g. low. high
+FAN_MODES = {
+    0: "standby",
+    1: "auto",
+    2: "low",
+    3: "medium",
+    4: "high",  # a.k.a. boost if timer on
+}
+FAN_RATE = "fan_rate"  # percentage, 0.0 - 1.0
