@@ -32,7 +32,7 @@ from .command import (
     QOS_TX_TIMEOUT,
     Command,
 )
-from .const import _PUZZ, HGI_DEVICE_ID, NON_DEVICE_ID, NUL_DEVICE_ID, __dev_mode__
+from .const import HGI_DEVICE_ID, NON_DEVICE_ID, NUL_DEVICE_ID, __dev_mode__
 from .exceptions import InvalidPacketError
 from .helpers import dt_now
 from .packet import Packet
@@ -40,7 +40,7 @@ from .protocol import create_protocol_factory
 from .schema import SERIAL_CONFIG_SCHEMA
 from .version import VERSION
 
-DEV_MODE = __dev_mode__  # and False
+DEV_MODE = __dev_mode__ and False
 
 _LOGGER = logging.getLogger(__name__)
 # _LOGGER.setLevel(logging.WARNING)  # INFO may have too much detail
@@ -85,25 +85,6 @@ VALID_CHARACTERS = printable  # "".join((ascii_letters, digits, ":-<*# "))
 # !FS - save autotune
 
 
-def _normalise(pkt_line: str, log_file: bool = False) -> str:
-    """Perform any packet line hacks, as required.
-
-    Goals:
-     - ensure an evofw3 provides the exact same output as a HGI80
-     - handle 'strange' packets (e.g. I/08:/0008)
-     - correct any historical design failures (e.g. puzzle packets with an index)
-    """
-
-    # psuedo-RAMSES-II packets...
-    if pkt_line[10:14] in (" 08:", " 31:") and pkt_line[-16:] == "* Checksum error":
-        pkt_line = pkt_line[:-17] + " # Checksum error (ignored)"
-        (_LOGGER.error if DEV_MODE else _LOGGER.warning)(
-            "Packet line has been normalised (0x01)"
-        )
-
-    return pkt_line.strip()
-
-
 def _str(value: ByteString) -> str:
     try:
         result = "".join(
@@ -112,11 +93,39 @@ def _str(value: ByteString) -> str:
             if c in VALID_CHARACTERS
         )
     except UnicodeDecodeError:
-        _LOGGER.warning("%s << Cant decode bytestream (ignoring)", value)
+        _LOGGER.warning("%s < Cant decode bytestream (ignoring)", value)
         return ""
     return result
 
 
+def _normalise(pkt_line: str) -> str:
+    """Perform any (transparent) frame-level hacks, as required at (near-)RF layer.
+
+    Goals:
+    - ensure an evofw3 provides the exact same output as a HGI80
+    - handle 'strange' packets (e.g. I/08:/0008)
+    """
+
+    # psuedo-RAMSES-II packets...
+    if pkt_line[10:14] in (" 08:", " 31:") and pkt_line[-16:] == "* Checksum error":
+        pkt_line = pkt_line[:-17] + " # Checksum error (ignored)"
+        _LOGGER.warning("Packet line has been normalised (0x01)")
+
+    return pkt_line.strip()
+
+
+def _regex_hack(pkt_line: str, regex_filters) -> str:
+    """Perform any packet hacks, as configured."""
+
+    for k, v in regex_filters.items():
+        try:
+            pkt_line = re.sub(k, v, pkt_line)
+        except re.error as exc:
+            _LOGGER.warning(f"{pkt_line} < issue with regex ({k}, {v}): {exc}")
+
+    return pkt_line
+
+
 class SerTransportRead(asyncio.ReadTransport):
     """Interface for a packet transport via a dict (saved state) or a file (pkt log)."""
 
@@ -330,7 +339,7 @@ class PacketProtocolBase(asyncio.Protocol):
                 "is strongly recommended)"
             )
 
-    # @functools.lru_cache(maxsize=128)
+    # @functools.lru_cache(maxsize=128)  # this will no longer work
     def _is_wanted(self, src_id, dst_id) -> Optional[bool]:
         """Parse the packet, return True if the packet is not to be filtered out.
 
@@ -393,8 +402,19 @@ class PacketProtocolBase(asyncio.Protocol):
                 self._callback(pkt)  # only wanted PKTs to the MSG transport's handler
             except InvalidPacketError as exc:
                 _LOGGER.error("%s < %s", pkt, exc)
-            except:  # noqa: E722  # TODO: remove broad-except
-                _LOGGER.exception("Exception in callback to message layer")
+            # except Exception as exc:  # noqa: E722, broad-except
+            except (
+                ArithmeticError,  # incl. ZeroDivisionError,
+                AssertionError,
+                AttributeError,
+                IndexError,
+                LookupError,  # incl. IndexError, KeyError
+                NameError,  # incl. UnboundLocalError
+                RuntimeError,  # incl. RecursionError
+                TypeError,
+                ValueError,
+            ) as exc:
+                _LOGGER.exception("%s < exception from msg layer: %s", pkt, exc)
 
     def _line_received(self, dtm: dt, line: str, raw_line: ByteString) -> None:
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
@@ -403,7 +423,12 @@ class PacketProtocolBase(asyncio.Protocol):
         self._hgi80[IS_INITIALIZED], was_initialized = True, self._hgi80[IS_INITIALIZED]
 
         try:
-            pkt = Packet.from_port(self._gwy, dtm, line, raw_line=raw_line)
+            pkt = Packet.from_port(
+                self._gwy,
+                dtm,
+                _regex_hack(line, self._gwy.config.use_regex.get("inbound", {})),
+                raw_line=raw_line,
+            )
 
         except InvalidPacketError as exc:
             if "# evofw" in line and self._hgi80[IS_EVOFW3] is None:
@@ -458,7 +483,12 @@ class PacketProtocolBase(asyncio.Protocol):
         # ):
         #     await asyncio.sleep(0.005)
 
-        data = bytes(data.encode("ascii"))
+        data = bytes(
+            _regex_hack(
+                data,
+                self._gwy.config.use_regex.get("outbound", {}),
+            ).encode("ascii")
+        )
 
         if _LOGGER.getEffectiveLevel() == logging.INFO:  # i.e. don't log for DEBUG
             _LOGGER.info("RF Tx:     %s", data)
@@ -527,7 +557,7 @@ class PacketProtocolPort(PacketProtocolBase):
         self._transport.write(bytes("!V\r\n".encode("ascii")))
 
         # add this to start of the pkt log, if any
-        if not self._disable_sending:
+        if not self._disable_sending:  # TODO: use a callback
             self._loop.create_task(self.send_data(Command._puzzle()))
 
         self.resume_writing()
@@ -563,7 +593,11 @@ class PacketProtocolRead(PacketProtocolBase):
     def _line_received(self, dtm: str, line: str, raw_line: str) -> None:
 
         try:
-            pkt = Packet.from_file(self._gwy, dtm, line)
+            pkt = Packet.from_file(
+                self._gwy,
+                dtm,
+                _regex_hack(line, self._gwy.config.use_regex.get("inbound", {})),
+            )
 
         except (InvalidPacketError, ValueError):  # VE from dt.fromisoformat()
             return
@@ -583,7 +617,7 @@ class PacketProtocolRead(PacketProtocolBase):
     def data_received(self, data: str) -> None:
         """Called when a packet line is received (from a log file)."""
         # _LOGGER.debug("PacketProtocolRead.data_received(%s)", data.rstrip())
-        self._line_received(data[:26], _normalise(data[27:], log_file=True), data)
+        self._line_received(data[:26], data[27:].strip(), data)
 
     def _dt_now(self) -> dt:
         try:
@@ -665,79 +699,21 @@ class PacketProtocolQos(PacketProtocolPort):
         Wraps the relevant function with QoS code.
         """
 
-        self._qos_received_v2(pkt)
+        self._qos_received(pkt)
 
         super()._pkt_received(pkt)
 
-    def _qos_received_v1(self, pkt: Packet) -> None:
-        """Perform any QoS functions on packets received from the transport."""
-
-        def _logger_rcvd(logger, message: str) -> None:
-            if self._qos_cmd is None:
-                wanted = None
-            elif self._tx_hdr:
-                wanted = self._tx_hdr
-            else:
-                wanted = self._rx_hdr
-
-            logger(
-                "PktProtocolQos.data_rcvd(rcvd=%s): boff=%s, want=%s, tout=%s: %s",
-                pkt._hdr or str(pkt),
-                self._backoff,
-                wanted,
-                self._timeout_full.isoformat(timespec="milliseconds"),
-                message,
-            )
-
-        if self._qos_cmd:
-
-            # NOTE: is the Tx pkt, and no response is expected
-            if pkt._hdr == self._tx_hdr and self._rx_hdr is None:
-                log_msg = "matched the Tx pkt (not wanting a Rx pkt) - now done"
-                self._qos_set_cmd(None)
-
-            # NOTE: is the Tx pkt, and a response *is* expected
-            elif pkt._hdr == self._tx_hdr:
-                # assert str(pkt)[4:] == str(self._qos_cmd), "Packets dont match"
-                log_msg = "matched the Tx pkt (now wanting a Rx pkt)"
-                self._tx_hdr = None
-
-            # NOTE: is the Tx pkt, but is a *duplicate* - we've already seen it!
-            elif pkt._hdr == self._qos_cmd.tx_header:
-                # assert str(pkt) == str(self._qos_cmd), "Packets dont match"
-                log_msg = "duplicated Tx pkt (still wanting the Rx pkt)"
-                self._qos_update_timeouts()  # TODO: increase backoff?
-
-            # NOTE: is the Rx pkt, and is a non-Null (expected) response
-            elif pkt._hdr == self._rx_hdr:
-                log_msg = "matched the Rx pkt - now done"
-                self._qos_set_cmd(None)
-
-            # NOTE: is not the expected pkt, but another pkt
-            else:
-                log_msg = (
-                    "unmatched pkt (still wanting a "
-                    + ("Tx" if self._tx_hdr else "Rx")
-                    + " pkt)"
-                )
-
-            self._qos_update_timeouts()
-            _logger_rcvd(_LOGGER.debug, f"CHECKED - {log_msg}")
-
-        else:  # TODO: no outstanding cmd - ?throttle down the backoff
-            # self._qos_update_timeouts()
-            _logger_rcvd(_LOGGER.debug, "XXXXXXX - ")
-
-    def _qos_received_v2(self, pkt: Packet) -> None:
+    def _qos_received(self, pkt: Packet) -> None:
         """Perform any QoS functions on packets received from the transport."""
 
         def logger_rcvd(message: str, wanted: Packet = None) -> None:
+            # Reserve the use of _LOGGER.info() for RF Tx/Rx logging
             if wanted:
-                logger = _LOGGER.warning if DEV_MODE else _LOGGER.info
+                logger = _LOGGER.warning if DEV_MODE else _LOGGER.debug
             elif message:
                 logger = _LOGGER.error if DEV_MODE else _LOGGER.warning
             else:
-                logger = _LOGGER.info if DEV_MODE else _LOGGER.debug
+                logger = _LOGGER.debug if DEV_MODE else _LOGGER.debug
                 message = "not wanting any pkt"
 
             logger(
@@ -795,77 +771,13 @@ class PacketProtocolQos(PacketProtocolPort):
 
         await super().send_data(cmd)
 
-        return await self._qos_send_data_v2(cmd)
+        return await self._qos_send_data(cmd)
 
-    async def _qos_send_data_v1(self, cmd: Command) -> None:
-        """Perform any QoS functions on packets sent to the transport."""
-
-        def _logger_send(logger, message: str) -> None:
-            logger(
-                "PktProtocolQos.send_data(sent=%s): boff=%s, want=%s, tout=%s: %s",
-                cmd.tx_header,
-                self._backoff,
-                self._tx_hdr or self._rx_hdr,
-                self._timeout_full.isoformat(timespec="milliseconds"),
-                message,
-            )
-
-        def _expired_cmd(cmd):
-            hdr, callback = cmd.tx_header, cmd.callback
-            if callback and not callback.get("expired"):
-                # see also: MsgTransport._pkt_receiver()
-                _LOGGER.error("PktProtocolQos.send_data(%s): Expired callback", hdr)
-                callback[FUNC](False, *callback.get(ARGS, tuple()))
-                callback["expired"] = not callback.get(DEAMON, False)  # HACK:
-
-        self._qos_set_cmd(cmd)
-        # self._tx_retries = 0
-        # self._tx_retry_limit = cmd.qos.get("retries", QOS_TX_RETRIES)
-        self._qos_update_timeouts()
-
-        while self._qos_cmd is not None:  # until sent (may need re-transmit) or expired
-            await asyncio.sleep(_QOS_POLL_INTERVAL)
-
-            if self._timeout_full > self._dt_now():
-                await asyncio.sleep(0.02)
-                # await self._send_data("")  # NOTE: untested - remove
-
-            elif self._qos_cmd is None:  # can be set to None by data_received
-                continue
-
-            elif self._tx_retries < self._tx_retry_limit:
-                # self._tx_hdr = cmd.tx_header
-                self._tx_retries += 1
-                if not self._qos_cmd.qos.get("disable_backoff", False):
-                    self._backoff = min(self._backoff + 1, _QOS_MAX_BACKOFF)
-                self._qos_update_timeouts()
-                await self._send_data(str(cmd))
-                _logger_send(
-                    (_LOGGER.warning if DEV_MODE else _LOGGER.debug),
-                    f"RE-SENT ({self._tx_retries}/{self._tx_retry_limit})",
-                )  # TODO: should be debug
-
-            else:
-                if self._qos_cmd.code != _PUZZ:  # HACK: why expired when shouldn't
-                    _logger_send(
-                        (_LOGGER.warning if DEV_MODE else _LOGGER.debug),
-                        f"EXPIRED ({self._tx_retries}/{self._tx_retry_limit})",
-                    )
-                    _expired_cmd(self._qos_cmd)
-
-                self._qos_set_cmd(None)
-                self._backoff = 0  # TODO: need a better system
-                break
-
-        else:
-            if self._timeout_half >= self._dt_now():
-                self._backoff = max(self._backoff - 1, 0)
-            _logger_send(_LOGGER.debug, "SENT OK")
-
-    async def _qos_send_data_v2(self, cmd: Command) -> None:
+    async def _qos_send_data(self, cmd: Command) -> None:
         """Perform any QoS functions on packets sent to the transport."""
 
         def logger_send(logger, message: str, wanted: Packet = None) -> None:
+            # Reserve the use of _LOGGER.info() for RF Tx/Rx logging
             logger(
                 "PktProtocolQos.send_data(sent=%s): boff=%s, want=%s, tout=%s: %s",
                 cmd.tx_header,
@@ -886,7 +798,7 @@ class PacketProtocolQos(PacketProtocolPort):
                 self._backoff = min(self._backoff + 1, _QOS_MAX_BACKOFF)
 
             if self._tx_retries < self._tx_retry_limit:
-                logger_send(_LOGGER.info, "TIMED_OUT_ (will retry)")
+                logger_send(_LOGGER.debug, "TIMED_OUT_ (will retry)")
 
                 self._tx_retries += 1
                 self._qos_update_timeouts()
