@@ -117,6 +117,7 @@ CONFIG_SCHEMA = vol.Schema(
         vol.Optional(ENFORCE_KNOWNLIST, default=None): vol.Any(None, bool),
         vol.Optional(USE_ALIASES, default=None): vol.Any(None, bool),
         vol.Optional(EVOFW_FLAG, default=None): vol.Any(None, str),
+        vol.Optional("use_regex", default={}): dict(),
     },
     extra=vol.ALLOW_EXTRA,  # TODO: remove for production
 )
@@ -247,6 +248,13 @@ def load_config(
     update_config(config, known_list, block_list)
     config = SimpleNamespace(**config)
 
+    # # TODO: remove
+    # config.use_regex.update(
+    #     {
+    #         "( 03:.* 03:.* (1060|2389|30C9) 003) ..": "\\1 00",
+    #     }
+    # )
+
     return (config, schema, known_list, block_list)
 
 
@@ -261,20 +269,26 @@ def update_config(config, known_list, block_list) -> dict:
         config[ENFORCE_KNOWNLIST] = False
 
     if config[ENFORCE_KNOWNLIST]:
-        _LOGGER.debug(
+        _LOGGER.info(
             f"The {KNOWN_LIST} will be used "
             f"as a whitelist (device_id filter), length = {len(known_list)}"
         )
+        _LOGGER.debug(f"known_list = {known_list}")
+
     elif block_list:
-        _LOGGER.debug(
+        _LOGGER.info(
             f"The {BLOCK_LIST} will be used "
             f"as a blacklist (device_id filter), length = {len(block_list)}"
         )
+        _LOGGER.debug(f"block_list = {block_list}")
+
     elif known_list:
         _LOGGER.warning(
             f"It is strongly recommended to use the {KNOWN_LIST} "
             f"as a whitelist (device_id filter), configure: {ENFORCE_KNOWNLIST} = True"
         )
+        _LOGGER.debug(f"known_list = {known_list}")
+
     else:
         _LOGGER.warning(
             f"It is strongly recommended to provide a {KNOWN_LIST}, and use it "
@@ -350,6 +364,9 @@ def load_system(gwy, ctl_id, schema) -> Tuple[dict, dict]:
         for dev_id in attrs.get(ATTR_DEVICES, []):
             _get_device(gwy, dev_id, ctl_id=ctl.id, domain_id=zone_idx)
 
+        if zone_type := attrs.get(ATTR_ZONE_TYPE):
+            zone._set_zone_type(zone_type)
+
     for dev_id in schema.get(ATTR_UFH_SYSTEM, {}).keys():  # UFH controllers
         _get_device(gwy, dev_id, ctl_id=ctl.id)  # , **_schema)
 
